﻿Imports BenchmarkDotNet.Attributes
Imports Microsoft.EntityFrameworkCore

Module Module1

    Sub Main()
        Console.ForegroundColor = ConsoleColor.Red
        Console.WriteLine($"===== {AppDomain.CurrentDomain.FriendlyName} =====")
        Console.WriteLine(String.Empty)
        Console.ResetColor()

        Start()


    End Sub


    Private Sub Start()
        Select Case WriteHomeMenu()
            Case 1
                Using ctx As New Context.PersonContext
                    Console.WriteLine("Lösche DB. Bitte warten...")
                    ctx.Database.EnsureDeleted()
                    Console.WriteLine("Erstelle DB. Bitte warten...")
                    ctx.Database.EnsureCreated()
                    Console.WriteLine("Schreibe 20.000 Testdatensätze in die Datenbank... Dies kann auch gerne ein bis zwei Minuten dauern")
                    ctx.Seed()
                    Console.WriteLine($"Fertig - {ctx.Person.Count} Datensätze in der Datenbank vorhanden.")
                    Console.WriteLine(String.Empty)
                    Start() : Exit Sub
                End Using
            Case 2
                StartStresstest1()
                Start() : Exit Sub
            Case 3
                StartStresstest2()
                Start() : Exit Sub
            Case 4
                StartStresstest3()
                Start() : Exit Sub
            Case 5
                StartStresstest4()
                Start() : Exit Sub
        End Select


        Console.Read()
    End Sub

    Private Sub StartStresstest4()
        Console.ResetColor()
        Console.ForegroundColor = ConsoleColor.Yellow
        Console.WriteLine("===== Update 3 Methoden. (1000 Datensätze) =====")
        Console.WriteLine($"EF im Tracking-Mode wo alle Entitäten in den Changetracker geladen, in einer Schleife aktualisiert und anschliessend gespeichert werden.")
        Console.WriteLine($"EF im NoTracking-Mode wo die Daten geladen und der EntityState für das Zip Property auf 'Modified' gesetzt wird. Anschliessend wird gespeichert.")
        Console.WriteLine($"EF mit ExecuteSQLCommand (UPDATE dbo.Person SET Zip = '99999' WHERE Id >= 1000 And Id < 2000)")
        Console.ResetColor()
        Console.WriteLine("Beliebige Taste drücken um zu starten...")
        Console.ReadKey()

        BenchmarkDotNet.Running.BenchmarkRunner.Run(New EfTest4().GetType)
        Console.WriteLine(String.Empty) : Console.WriteLine(String.Empty)
        Console.ResetColor()
    End Sub

    Private Sub StartStresstest3()
        Console.ResetColor()
        Console.ForegroundColor = ConsoleColor.Yellow
        Console.WriteLine("===== Select mit 4 Methoden. (10000 Datensätze) =====")
        Console.WriteLine($"EF im Tracking-Mode wo alle Entitäten in den Changetracker geladen werden. {Environment.NewLine} (ctx.Person.Skip(10).Take(10000).Where(Function(p) p.FirstName.StartsWith(""Vor"") AndAlso p.LastName.StartsWith(""Nach"")).OrderBy(Function(o) o.Id).AsTracking.Load){Environment.NewLine}{Environment.NewLine}")
        Console.WriteLine($"EF im NoTracking-Mode wo der Changetracker außen vor bleibt. {Environment.NewLine}(ctx.Person.Skip(10).Take(10000).Where(Function(p) p.FirstName.StartsWith(""Vor"") AndAlso p.LastName.StartsWith(""Nach"")).OrderBy(Function(o) o.Id).AsTracking.Load){Environment.NewLine}{Environment.NewLine}")
        Console.WriteLine($"EF mit FromSql im Tracking-Mode {Environment.NewLine}(SELECT * FROM dbo.Person WHERE FirstName LIKE 'Vor%' AND LastName LIKE 'Nach%' ORDER BY id OFFSET 10 ROWS FETCH NEXT 10000 ROWS ONLY)")
        Console.WriteLine($"EF mit FromSql im NoTracking-Mode {Environment.NewLine}(SELECT * FROM dbo.Person WHERE FirstName LIKE 'Vor%' AND LastName LIKE 'Nach%' ORDER BY id OFFSET 10 ROWS FETCH NEXT 10000 ROWS ONLY)")
        Console.ResetColor()
        Console.WriteLine("Beliebige Taste drücken um zu starten...")
        Console.ReadKey()

        BenchmarkDotNet.Running.BenchmarkRunner.Run(New EfTest3().GetType)
        Console.WriteLine(String.Empty) : Console.WriteLine(String.Empty)
        Console.ResetColor()
    End Sub

    Private Sub StartStresstest2()
        Console.ResetColor()
        Console.ForegroundColor = ConsoleColor.Yellow
        Console.WriteLine("===== Einfaches Select mit 4 Methoden. (10000 Datensätze) =====")
        Console.WriteLine("EF im Tracking-Mode wo alle Entitäten in den Changetracker geladen werden. (ctx.Person.Take(10000).AsTracking.Load)")
        Console.WriteLine("EF im NoTracking-Mode wo der Changetracker außen vor bleibt. (ctx.Person.Take(10000).AsNoTracking.Load)")
        Console.WriteLine("EF mit FromSql im Tracking-Mode (SELECT TOP 10000 * FROM Person)")
        Console.WriteLine("EF mit FromSql im NoTracking-Mode (SELECT TOP 10000 * FROM Person)")
        Console.ResetColor()
        Console.WriteLine("Beliebige Taste drücken um zu starten...")
        Console.ReadKey()

        BenchmarkDotNet.Running.BenchmarkRunner.Run(New EfTest2().GetType)
        Console.WriteLine(String.Empty) : Console.WriteLine(String.Empty)
        Console.ResetColor()
    End Sub

    Private Sub StartStresstest1()
        Console.ResetColor()
        Console.ForegroundColor = ConsoleColor.Yellow
        Console.WriteLine("===== Einfaches Select mit 4 Methoden. (2000 Datensätze) =====")
        Console.WriteLine("EF im Tracking-Mode wo alle Entitäten in den Changetracker geladen werden. (ctx.Person.Take(2000).AsTracking.ToList)")
        Console.WriteLine("EF im NoTracking-Mode wo der Changetracker außen vor bleibt. (ctx.Person.Take(2000).AsNoTracking.ToList)")
        Console.WriteLine("EF mit FromSql im Tracking-Mode (SELECT TOP 2000 * FROM Person)")
        Console.WriteLine("EF mit FromSql im NoTracking-Mode (SELECT TOP 2000 * FROM Person)")
        Console.ResetColor()
        Console.WriteLine("Beliebige Taste drücken um zu starten...")
        Console.ReadKey()

        BenchmarkDotNet.Running.BenchmarkRunner.Run(New EfTest1().GetType)
        Console.WriteLine(String.Empty) : Console.WriteLine(String.Empty)
        Console.ResetColor()
    End Sub

    '<Benchmark>
    'Private Sub DoBenchmarkTest1(rowNumbers As Integer)
    '    Using ctx As New Context.PersonContext
    '        For i As Integer = 1 To 20
    '            ctx.Person.Take(rowNumbers).AsTracking.ToList
    '        Next
    '    End Using
    'End Sub



    Public Function WriteHomeMenu() As Integer
        Dim choosenMenuPointInt As Integer = 0

        Do While choosenMenuPointInt = 0
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine($"Was möchtest du als nächstes tun?")
            Console.ResetColor()
            Console.WriteLine("1.) Datenbank löschen und neu anlegen sowie mit Daten befüllen")
            Console.WriteLine("2.) Performancetest für Select auf 2000 Datensätze")
            Console.WriteLine("3.) Performancetest für Select auf 10000 Datensätze")
            Console.WriteLine("4.) Performancetest für Select auf 10000 Datensätze mit Bedingungen")
            Console.WriteLine("5.) Performancetest für Update von 1000 Datensätzen")
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.WriteLine(String.Empty)
            Console.Write("Bitte wähle eine Nummer aus dem Hauptmenü:")
            Console.ResetColor()

            Dim choosenMenuPointString = Console.ReadLine

            If Not Integer.TryParse(choosenMenuPointString, choosenMenuPointInt) OrElse (choosenMenuPointInt < 1 Or choosenMenuPointInt > 5) Then
                WriteError($"'{choosenMenuPointString}' ist keine gültige Eingabe. Bitte versuche es nochmals.")
                choosenMenuPointInt = 0
            End If
        Loop
        Console.WriteLine(String.Empty)
        Console.WriteLine(String.Empty)
        Return choosenMenuPointInt
    End Function

    Public Sub WriteError(errText As String)
        Console.WriteLine(String.Empty)
        Console.ForegroundColor = ConsoleColor.Red
        Console.WriteLine($"Fehler: {errText}")
        Console.ResetColor()
        Console.WriteLine(String.Empty)
    End Sub


    Private Function AskForNumber(question As String, allowedMinValue As Integer, allowedMaxValue As Integer) As Integer
        If String.IsNullOrEmpty(question) Then Throw New ArgumentNullException(NameOf(question))
        If allowedMinValue < 1 Then Throw New ArgumentOutOfRangeException(NameOf(allowedMinValue), "Der Wert darf nicht unter 1 liegen!")
        If allowedMaxValue < 1 Then Throw New ArgumentOutOfRangeException(NameOf(allowedMaxValue), "Der Wert darf nicht unter 0 liegen!")

        Dim choosenInt As Integer = 0

        Do While choosenInt = 0
            Console.ForegroundColor = ConsoleColor.Yellow
            Console.Write(question)
            Console.ResetColor()

            Dim choosenString = Console.ReadLine

            If Not Integer.TryParse(choosenString, choosenInt) OrElse (choosenInt < allowedMinValue Or choosenInt > allowedMaxValue) Then
                WriteError($"'{choosenString}' ist keine gültige Eingabe. Bitte versuche es nochmals.")
                choosenInt = 0
            End If
        Loop
        Return choosenInt
    End Function

End Module
